/**
 * Copyright narutohyper ( http://wonderfl.net/user/narutohyper )
 * MIT License ( http://www.opensource.org/licenses/mit-license.php )
 * Downloaded from: http://wonderfl.net/c/gGFI
 */

// forked from narutohyper's Alternativa3D 7.5 Template
package
{
    import alternativ7.engine3d.controllers.SimpleObjectController;
    import alternativ7.engine3d.core.Object3DContainer;
    import alternativ7.engine3d.loaders.MaterialLoader;
    import alternativ7.engine3d.materials.TextureMaterial;
    
    import flash.display.Sprite;
    import flash.events.Event;
    import flash.system.LoaderContext;
    
    /**
     * Alternativa3D 7.6
     *
     * PolygonPlane Primitive
     *
     * @author narutohyper
     */
    [SWF(backgroundColor="#000000", frameRate="100", width="800", height="600")]
    public class Main extends Sprite
    {
        public function Main():void
        {
        
            if (stage) init();
            else addEventListener(Event.ADDED_TO_STAGE, init);
        }

        
        private function init(e:Event=null):void {
            removeEventListener(Event.ADDED_TO_STAGE, init);
        
            //AlternativaTemplate作成
            var scene:AlternativaTemplate = new AlternativaTemplate(this);

            
            //まとめてcontrolするために、core.Object3DContainerを作成し、各Objectを入れ子にする
            var animeObject:Object3DContainer = new Object3DContainer;
            scene.container.addChild(animeObject);
            
            
            //外部読み込み画像によるmaterial
            var textureMaterial:TextureMaterial = new TextureMaterial(null, false, true);
            textureMaterial.diffuseMapURL = 'http://assets.wonderfl.net/images/related_images/e/e9/e987/e9876c502c9e9693205335e106de6baa8865c1a6m';
            var textures:Vector.<TextureMaterial> = new Vector.<TextureMaterial>(1);
            textures[0] = textureMaterial;

            var context:LoaderContext = new LoaderContext(true);
            var materialLoader:MaterialLoader = new MaterialLoader();
            materialLoader.load(textures,context);


            //オリジナルプリミティブ、PolygonPlaneを使用して、様々な図形を作成

            
            //三角形なPlaneを作成
            var meshTriangle:PolygonPlane = new PolygonPlane(200,3,-90);
            meshTriangle.setMaterialToAllFaces(textureMaterial);
            animeObject.addChild(meshTriangle);
            meshTriangle.x=-500;
            meshTriangle.y=-500;
            
            
            //四角形なPlaneを作成
            var meshSquare:PolygonPlane = new PolygonPlane(200,4,-135);
            meshSquare.setMaterialToAllFaces(textureMaterial);
            animeObject.addChild(meshSquare);
            meshSquare.x = -500;
            meshSquare.y = 500;


            //多角形なPlaneを作成
            var meshHexagon:PolygonPlane = new PolygonPlane(200,6,-120);

            meshHexagon.setMaterialToAllFaces(textureMaterial);
            animeObject.addChild(meshHexagon);
            meshHexagon.x = 500;
            meshHexagon.y = -500;


            //星型
            //凸凹型ポリゴンも、生成する
            var meshStar:PolygonPlane = new PolygonPlane(200,10,-90+36,true);

            meshStar.setMaterialToAllFaces(textureMaterial);
            animeObject.addChild(meshStar);
            meshStar.x = 500;
            meshStar.y = 500;
            
            
            //円というか、36角形
            var meshCircle:PolygonPlane = new PolygonPlane(200,36);

            meshCircle.setMaterialToAllFaces(textureMaterial);
            animeObject.addChild(meshCircle);
            meshCircle.x = 0;
            meshCircle.y = 0;




            //カメラの調整
            //カメラはControllerに関連付けている為、x,y,zで直接位置を指定できないので
            //SimpleController.setObjectPosXYZを使用
            scene.cameraController.setObjectPosXYZ(0, -1500, 500);
            scene.cameraController.lookAtXYZ(0, 0, 0);
            //ObjectControllerの作成
            //MouseDragで、Objectを回転させる為のController
            var objectController:SimpleObjectController = new SimpleObjectController(stage,animeObject,100);
            objectController.unbindAll();
            
            scene.onPreRender = function():void {
                objectController.update();
            }
            
            //描画開始
            scene.startRendering();

        }

    }
    
}





import alternativ7.engine3d.objects.Mesh;
import alternativ7.engine3d.materials.Material;
import alternativ7.engine3d.core.Vertex;
/**
 * 多角形PlaneMeshを作成するクラス
 * @author narutohyper
 */
class PolygonPlane extends Mesh
{
    
    public function PolygonPlane(radius:Number = 100, vertexNum:Number = 3, startVertexAngle:Number=-90 , star:Boolean = false, twoFace:Boolean = true, reverse:Boolean = false, frontMaterial:Material = null, backMaterial:Material = null)
    {
        var vertex:Vector.<Vertex> = new Vector.<Vertex>(vertexNum);
        var RADIAN:Number = Math.PI / 180;
        var sr:Number = startVertexAngle;
        var pitch:Number = 360/vertexNum;
        var angle:Number;
        var i:uint;
        var length:Number = radius;

        var tw:Number = Math.cos(0 * RADIAN) * length * 2;
        var th:Number = Math.sin(90 * RADIAN) * length * 2;
        
        trace(tw,th);
        
        var tx:Number;
        var ty:Number;
        var tu:Number;
        var tv:Number;
        var v:Vertex;
        
        for (i = 0; i < vertexNum; i++) {
            angle = (sr + (pitch * i)) * RADIAN;
            if (star) {
                if (i && i % 2) {
                    tx = Math.cos(angle) * length;
                    ty = Math.sin(angle) * length;
                } else {
                    tx = Math.cos(angle) * length/2;
                    ty = Math.sin(angle) * length/2;
                }
            } else {
                tx = Math.cos(angle) * length;
                ty = Math.sin(angle) * length;
            }
            tu = (tx + tw / 2) / tw;
            tv = (ty + th / 2) / th;
            
            vertex[i] = addVertex(tx, ty, 0, tu, tv, 'v'+i);
        }
        if (!reverse) {
            addFace(vertex,frontMaterial, 'faceFront');
            if (twoFace) {
                vertex.reverse();
                v = vertex.shift()
                vertex.push(v);
                addFace(vertex,backMaterial, 'faceBack');
            }
        } else {
            if (twoFace) {
                addFace(vertex,frontMaterial, 'faceFront');
            }
            vertex.reverse();
            v = vertex.shift()
            vertex.push(v);
            addFace(vertex,backMaterial, 'faceBack');
            
        }
        calculateNormals();
        x = 0;
        y = 0;
    }
    
}


/**
 * BasicTemplate for Alternativa3D 7.5
 * Alternativa3D 7.5を扱いやすくするためのテンプレートです
 * @author narutohyper & clockmaker
 *
 */
import alternativ7.engine3d.containers.BSPContainer;
import alternativ7.engine3d.containers.ConflictContainer;
import alternativ7.engine3d.containers.DistanceSortContainer;
import alternativ7.engine3d.containers.KDContainer;
import alternativ7.engine3d.containers.LODContainer;
import alternativ7.engine3d.controllers.SimpleObjectController;
import alternativ7.engine3d.core.Camera3D;
import alternativ7.engine3d.core.Object3DContainer;
import alternativ7.engine3d.core.View;
import flash.display.DisplayObject;

import flash.display.DisplayObjectContainer;
import flash.display.Sprite;
import flash.display.StageAlign;
import flash.display.StageQuality;
import flash.display.StageScaleMode;

import flash.events.Event;

class AlternativaTemplate extends Sprite
{
    /**
     * 子オブジェクトを最適な方法でソートするコンテナ
     * (ConflictContainer)
     */
    public static const CONFLICT:String = 'conflict';
    /**
     * 子オブジェクトをBSP(バイナリ空間分割法)によってソートするコンテナ
     * (BSPContainer)
     */
    public static const BSP:String = 'bsp';
    
    /**
     * 子オブジェクトをカメラからのZ値でソートするコンテナ
     * (DistanceSortContainer)
     */
    public static const ZSORT:String = 'zsort';
    /**
     * KDツリー(http://ja.wikipedia.org/wiki/Kd%E6%9C%A8)によってソートするコンテナ
     * (KDContainer)
     */
    public static const KD:String = 'kd';
    /**
     * detalizationと子オブジェクトの距離でソートするコンテナ（詳細は調査中）
     * (LODContainer)
     */
    public static const LOD:String = 'lod';
    
    /**
     * 3dオブジェクト格納するコンテナインスタンス。
     */
    public var container:Object3DContainer;

    /**
     * ビューインスタンスです。
     */
    public var view:View;
    
    /**
     * カメラインスタンスです。
     */
    public var camera:Camera3D;
    
    /**
     * カメラコントローラーです。
     */
    public var cameraController:SimpleObjectController;
    
    private var _mc:DisplayObjectContainer;
    private var _viewWidth:int;
    private var _viewHeight:int;
    private var _scaleToStage:Boolean;
    private var _containerType:String;
    
    /**
     * 新しい Alternativa3DTemplate インスタンスを作成します。
     * @param    mc
     * @param    containerType
     * @param    viewWidth
     * @param    viewHeight
     * @param    scaleToStage
     */
    public function AlternativaTemplate(mc:DisplayObjectContainer,containerType:String=CONFLICT,viewWidth:int=640, viewHeight:int=480, scaleToStage:Boolean = true)
    {
        
        _mc = mc;
        _mc.addChild(this);

        _containerType = containerType;
        _viewWidth = viewWidth;
        _viewHeight = viewHeight;
        _scaleToStage = scaleToStage;
        
        if (stage) init();
        else addEventListener(Event.ADDED_TO_STAGE, init);
    }


    /**
     * 初期化されたときに実行されるイベントです。
     * 初期化時に実行したい処理をオーバーライドして記述します。
     */
    protected function atInit():void {}


    /**
     * Event.ENTER_FRAME 時に実行されるレンダリングのイベントです。
     * レンダリング前に実行したい処理をオーバーライドして記述します。
     */
    protected function atPreRender():void {}
    
    /**
     * Event.ENTER_FRAME 時に実行されるレンダリングのイベントです。
     * レンダリング前に実行したい処理をオーバーライドして記述します。
     */
    private var _onPreRender:Function = function():void{};
    public function get onPreRender():Function { return _onPreRender; }
    public function set onPreRender(value:Function):void
    {
        _onPreRender = value;
    }
    
    /**
     * Event.ENTER_FRAME 時に実行されるレンダリングのイベントです。
     * レンダリング後に実行したい処理をオーバーライドして記述します。
     */
    protected function atPostRender():void {}
    
    /**
     * Event.ENTER_FRAME 時に実行されるレンダリングのイベントです。
     * レンダリング後に実行したい処理を記述します。
     */
    private var _onPostRender:Function = function():void{};
    public function get onPostRender():Function { return _onPostRender; }
    public function set onPostRender(value:Function):void
    {
        _onPostRender = value;
    }
    
    
    /**
     * レンダリングを開始します。
     */
    public function startRendering():void
    {
        addEventListener(Event.ENTER_FRAME, onRenderTick);
    }

    /**
    * レンダリングを停止します。
    */
    public function stopRendering():void
    {
        removeEventListener(Event.ENTER_FRAME, onRenderTick);
    }

    /**
     * シングルレンダリング(レンダリングを一回だけ)を実行します。
     */
    public function singleRender():void
    {
        onRenderTick();
    }

    /**
     * @private
     */
    private function init(e:Event = null):void
    {
        removeEventListener(Event.ADDED_TO_STAGE, init);
        // entry point
        stage.scaleMode = StageScaleMode.NO_SCALE;
        stage.align = StageAlign.TOP_LEFT;
        stage.quality = StageQuality.HIGH;
        
        //Root objectの作成
        if (_containerType == CONFLICT) {
            container = new ConflictContainer();
        } else if (_containerType == BSP) {
            container = new BSPContainer();
        } else if (_containerType == ZSORT) {
            container = new DistanceSortContainer();
        } else if (_containerType == KD) {
            container = new KDContainer();
        } else if (_containerType == LOD) {
            container = new LODContainer();
        }
        //Viewの作成
        view = new View(stage.stageWidth, stage.stageHeight);
        _mc.addChild(view);

        //cameraの作成
        camera = new Camera3D();
        camera.view = view;
        camera.x = 0;
        camera.y = -500;
        camera.z = 0;
        container.addChild(camera);
        
        // Camera controller
        cameraController = new SimpleObjectController(stage, camera, 10);
        cameraController.mouseSensitivity = 0;
        cameraController.unbindAll();
        cameraController.lookAtXYZ(0, 0, 0);
        
        onResize();
        stage.addEventListener(Event.RESIZE, onResize);
        
        atInit();
    }
    
    /**
     * @private
     */    
    private function onResize(e:Event = null):void 
    {
        if (_scaleToStage)
        {
            view.width = stage.stageWidth;
            view.height = stage.stageHeight;
        } 
        else
        {
            view.width = _viewWidth;
            view.height = _viewHeight;
        }
    }
    
    /**
     * @private
     */    
    private function onRenderTick(e:Event = null):void 
    {
        atPreRender();
        _onPreRender();
        cameraController.update();
        camera.render();
        atPostRender();
        _onPostRender();
    }
            
    
}

